package Chart::Palette;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Chart::Palette - Short module description

=head1 SYNOPSIS

 use Chart::Palette;

 ##
 # Either use static class methods:
 ##

 # Translate color name to RGB hexadecimal representation
 my $rgb = Chart::Palette->as_rgb('snow');        # $rgb = 'FFFAFA';
 my $rgb = Chart::Palette->as_rgb('FFFAFA');      # $rgb = 'FFFAFA';

 # Translate RGB hexadecimal representation to color name
 my $color = Chart::Palette->as_string('FFFAFA'); # $color = 'snow';
 my $color = Chart::Palette->as_string('snow');   # $color = 'snow';

 # Fetch random color
 my ($color, $rgb) = Chart::Palette->random_color();

 # Fetch random and unique color
 my ($color, $rgb) = Chart::Palette->random_unique_color();

 # Fetch random and unique color and give a list of already used and therefore
 # forbidden colors
 my ($color, $rgb) = Chart::Palette->random_unique_color('red','green','blue');
 my ($color, $rgb) = Chart::Palette->random_unique_color('FF0000','FFFF00','0000FF');

 # Fetch random color palette of given length (number of colors)
 my @palette = Chart::Palette->random_palette(10);

 # Fetch next color from static (hardcoded) palette
 my ($color, $rgb) = Chart::Palette->color();

 # Fetch next unique color from static (hardcoded) palette
 my ($color, $rgb) = Chart::Palette->unique_color();

 # Fetch next unique color from static (hardcoded) palette and give a list of
 # already used and therefore forbidden colors
 my ($color, $rgb) = Chart::Palette->unique_color('red','green','blue');
 my ($color, $rgb) = Chart::Palette->unique_color('FF0000','FFFF00','0000FF');

 # Fetch color palette (chunk of static palette of given lenght)
 my @palette = Chart::Palette->palette(10);

 ##
 # Or create instance and use object methods:
 ##

 my $instance = Chart::Palette->new();

 my $rgb = $instance->as_rgb('snow');        # $rgb = 'FFFAFA';
 my $rgb = $instance->as_rgb('FFFAFA');      # $rgb = 'FFFAFA';

 my $color = $instance->as_string('FFFAFA'); # $color = 'snow';
 my $color = $instance->as_string('snow');   # $color = 'snow';

 my ($color, $rgb) = $instance->random_color();

 my ($color, $rgb) = $instance->random_unique_color();
 my ($color, $rgb) = $instance->random_unique_color('red','green','blue');
 my ($color, $rgb) = $instance->random_unique_color('FF0000','FFFF00','0000FF');

 my @palette = $instance->random_palette(10);

 my ($color, $rgb) = $instance->color();

 my ($color, $rgb) = $instance->unique_color();
 my ($color, $rgb) = $instance->unique_color('red','green','blue');
 my ($color, $rgb) = $instance->unique_color('FF0000','FFFF00','0000FF');

 my @palette = $instance->palette(10);

=head1 DESCRIPTION

Extensive module description.

=head1 AUTHOR

Jan Mach
Cesnet, z.s.p.o
jan.mach@cesnet.cz
http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use List::Util 'shuffle';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#
my @COLOR_PALETTE; # Color palette
my $DFLT_COLORS;

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;

    # Great tool for choosing colors: http://colllor.com/
    @COLOR_PALETTE = (
        #['#7cb5ec','#94c3f0'],
        #['#434348','#4e4e55'],
        #['#90ed7d','#a3f093'],
        #['#f7a35c','#f9b177'],
        #['#8085e9','#979bed'],
        #['#303f9f','#3444ad'],
        #['#3f51b5','#6877ca'],
        #['#c5cae9','#e1e4f4'],
        #['#ff5252','#ff7070'],
        #['#212121','#292929'],
        #['#727272','#7a7a7a'],
        #['#b6b6b6','#c2c2c2'],
        #['#8bc34a','#93c757'],
        #['#ecd078','#f0da94'],
        #['#d95b43','#e07b67'],
        #['#c02942','#d43550'],
        #['#542437','#72314b'],
        #['#53777a','#638e92'],
        #['#ffb7c5','#ffd6de'],
        #['#c70025','#e00029'],
        #['#424b54','#48525b'],
        #['#f3cec8','#f8e1de'],
        #['#e8a317','#ebad33'],
        #['#f7e294','#f9eab3'],
        #['#cb4d3f','#cf5c4f'],
        ['#1f77b4'],
        ['#aec7e8'],
        ['#ff7f0e'],
        ['#ffbb78'],
        ['#2ca02c'],
        ['#98df8a'],
        ['#d62728'],
        ['#ff9896'],
        ['#9467bd'],
        ['#c5b0d5'],
        ['#8c564b'],
        ['#c49c94'],
        ['#e377c2'],
        ['#f7b6d2'],
        ['#7f7f7f'],
        ['#c7c7c7'],
        ['#bcbd22'],
        ['#dbdb8d'],
        ['#17becf'],
        ['#9edae5'],
    );
    $DFLT_COLORS = ['#000000','#010101'];
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC ,STATIC]

  Usage     : my $instance = Chart::Palette->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Chart::Palette reference on success, croaks on failure
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method invoked on object" if blessed($class);
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item palette(;$) [PUBLIC, HYBRID]

  Usage     : my @palette = $palette->palette($size); OR
              my @palette = Chart::Palette->palette($size);
  Purpose   : Fetch static color palette of given size
  Arguments : INTEGER $size - number of colors in the palette [OPTIONAL]
  Returns   : ARRAY of ARRAYS containing colors:
                STRING $rgb - color in RGB format (6 hexadecimal characters)
                STRING $rgb_light - lighter color in RGB format (6 hexadecimal characters)
  Throws    : Croaks, if requested palette size is too big
  Comments  : Hybrid method, may be called as class, or instance method

=cut

sub palette(;$)
{
    my $invocant = shift;
    my ($size,) = @_;
    confess "Requested palette size is too big" if $size and int($size) and int($size) > scalar(@COLOR_PALETTE);

    if ($size) {
        return [@COLOR_PALETTE[0 .. --$size]];
    }
    else {
        return [@COLOR_PALETTE];
    }
}

=item color($) [PUBLIC, HYBRID]

  Usage     : my @palette = $palette->color($inder); OR
              my @palette = Chart::Palette->color($index);
  Purpose   : Fetch static color from palette on given index
  Arguments : INTEGER $index -  [MANDATORY]
  Returns   : ARRAY of containing colors:
                STRING $rgb - color in RGB format (6 hexadecimal characters)
                STRING $rgb_light - lighter color in RGB format (6 hexadecimal characters)
  Throws    : Croaks, if requested palette size is too big
  Comments  : Hybrid method, may be called as class, or instance method

=cut

sub color($)
{
    my $invocant = shift;
    my ($index,) = @_;

    if ($index) {
        return $COLOR_PALETTE[($index-1)];
    }
    else {
        return $DFLT_COLORS;
    }
}

sub color_m($)
{
    my $invocant = shift;
    my ($index,) = @_;

    my $l = 0+@COLOR_PALETTE;
    if ($index) {
        return $COLOR_PALETTE[(($index-1)%$l)];
    }
    else {
        return $DFLT_COLORS;
    }
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Chart::Palette instance
#  Arguments :
#  Returns   : Chart::Palette reference
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object" unless blessed($self);
    return $self;
}

=pod

=back

=cut

1;
