package Mentat::Processor::Engine::Validator;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine::Validator - Engine performing ALERT message validation

=head1 SYNOPSIS

 use Mentat::Processor::Engine::Validator;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw(time);
use Digest::MD5 qw(md5_hex);

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:c_general);
use Mentat::Processor::Engine;
use Mentat::Message::Validator::IDEA;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Engine');
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item eh_process($$) [PUBLIC]

  Usage     : Emit/execute event 'message_hello' with appropriate arguments
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub eh_process($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;
    confess "Message must be a 'Mentat::Processor::Message' implementation" unless blessed($message) and $message->isa('Mentat::Processor::Message');

    # Retrieve the encapsulated IDEA message
    my $alert_message = $message->alert_obj;
    my $alert_class = $alert_message->class;

    unless ($self->{VALIDATORS}->{$alert_class}) {
        $self->statistics->{'message_errors_nonvalidatable'}++;
        Log::Loger->error(__PACKAGE__, "[ENG:Validator] Validator for message class '$alert_class' does not exist");
        
        $self->emit_event('message_banish', $message);
        return EC_STOP;
    }

    # If there was some problem during ALERT schema validation, store the message to the error folder
    if (my $error = $self->{VALIDATORS}->{$alert_class}->validate_s($alert_message)) {
        $self->statistics->{'message_errors_schema'}++;
        Log::Loger->error(__PACKAGE__, "[ENG:Validator] Message triggered schema error");

        $self->emit_event('message_banish', $message);
        return EC_STOP;
    }
    return (EC_CONTINUE, $message);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Engine instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    die "[ENG:Validator] Constructor expects configurations as argument\n" unless %config;
    die "[ENG:Validator] Validator engine configuration 'schema_file_idea' is missing\n" unless $config{schema_file_idea};
    die "[ENG:Validator] Validator engine configuration 'error_folder' is missing\n" unless $config{error_folder};

    Log::Loger->info(__PACKAGE__, "[ENG:Validator] Using schema file '$config{schema_file_idea}' for IDEA schema validations");
    Log::Loger->info(__PACKAGE__, "[ENG:Validator] Using folder '$config{error_folder}' as error folder");

    $self->{VALIDATORS} = {
            'idea'  => Mentat::Message::Validator::IDEA->new($config{schema_file_idea}),
        };
    $self->{ERROR_FOLDER} = $config{error_folder};

    return $self;
}

=pod

=back

=cut

1;
